PROGRAM pivchk

!  THIS PROGRAM ILLUSTRATES THE USE OF NSPIV BY SOLVING THE
!  SYSTEM OF LINEAR EQUATIONS

!       A X = B

!  WITH A AN NG X NG BLOCK TRIDIAGONAL MATRIX, WITH NG X NG BLOCKS.
!  THE DIAGONAL BLOCKS OF A ARE LOWER BI-DIAGONAL (ENTRIES ARE 4.0
!  ON THE DIAGONAL, -1.0 ON THE SUBDIAGONAL), AND THE OFF-DIAGONAL
!  BLOCKS OF A ARE DIAGONAL (ENTRIES ARE -1.0 IN THE LOWER TRIANGLE,
!  -1.5 IN THE UPPER TRIANGLE.)  X IS CHOSEN TO BE A VECTOR
!  OF ALL ONES, AND B IS COMPUTED ACCORDINGLY.

USE Sparse_Gauss
IMPLICIT NONE

INTEGER    :: ia(101), ja(400), r(100), c(100), ic(100)
INTEGER    :: i, iaptr, ierr, j, k
REAL (dp)  :: a(400), b(100), x(100)
REAL (dp)  :: bk, errmax
INTEGER, PARAMETER  :: maxn = 395, ng = 10, n = 100

!  SET UP PROBLEM

k = 1
ia(1) = 1
iaptr = 1
DO  i = 1, ng
  DO  j = 1, ng
    bk = 0.
    IF (i /= 1) THEN
      ja(iaptr) = k - ng
      a(iaptr) = -1.
      bk = bk - 1.
      iaptr = iaptr + 1
    END IF
    IF (j /= 1) THEN
      ja(iaptr) = k - 1
      a(iaptr) = -1.
      bk = bk - 1.
      iaptr = iaptr + 1
    END IF
    ja(iaptr) = k
    a(iaptr) = 4.
    bk = bk + 4.
    iaptr = iaptr + 1
    IF (i /= ng) THEN
      ja(iaptr) = k + ng
      a(iaptr) = -1.5
      bk = bk - 1.5
      iaptr = iaptr + 1
    END IF
    b(k) = bk
    k = k + 1
    ia(k) = iaptr
  END DO
END DO

!  CALL PREORD TO ORDER ROWS OF A BY INCREASING NUMBERS OF NONZEROES

CALL preord(n, ia, r, c, ic)

!  CALL NSPIV TO SOLVE SYSTEM

CALL nspiv(n, ia, ja, a, b, maxn, r, c, ic, x, ierr)
WRITE (*, 5000) ierr

!  Calculate the maximum error in the solution vector (should be 1's)
errmax = 0.0_dp
DO i = 1, n
  errmax = MAX(errmax, ABS(x(i) - 1.0_dp))
END DO
WRITE(*, '(a, g12.4)') ' Largest error in the solution vector =', errmax

!  CALL RESCHK TO COMPUTE MAX-NORM AND 2-NORM OF RESIDUAL

CALL reschk(n, ia, ja, a, b, x)

STOP
5000 FORMAT (' IERR = ', i10, '  OK if positive')
END PROGRAM pivchk
