PROGRAM PIVCHK

!  THIS PROGRAM ILLUSTRATES THE USE OF NSPIV BY SOLVING THE
!  SYSTEM OF LINEAR EQUATIONS

!       A X = B

!  WHERE A IS A BANDED 192 X 192 MATRIX OF BANDWIDTH 20.  X IS
!  CHOSEN TO BE A VECTOR OF ALL ONES, AND B IS COMPUTED ACCORDINGLY.

USE Sparse_Gauss
IMPLICIT NONE

INTEGER    :: ia(193), ja(3500), r(192), c(192), ic(192)
INTEGER    :: i, ierr, ipartj
REAL (dp)  :: a(3500), b(192), errmax, x(192), rtemp(7500)
INTEGER, PARAMETER  :: maxn = 7000, n = 192

!  CALL GENPRB TO SET UP PROBLEM

CALL genprb(n, ia, ja, a, rtemp, b)

!  CALL PREORD TO ORDER ROWS OF A BY INCREASING NUMBERS OF NONZEROES

CALL preord(n, ia, r, c, ic)

!  CALL NSPIV TO SOLVE SYSTEM

CALL nspiv(n, ia, ja, a, b, maxn, r, c, ic, x, ierr)
WRITE (*, 5000) ierr

!  Calculate the maximum error in the solution vector (should be 1's)
errmax = 0.0_dp
DO i = 1, n
  errmax = MAX(errmax, ABS(x(i) - 1.0_dp))
END DO
WRITE(*, '(a, g12.4)') ' Largest error in the solution vector =', errmax

!  CALL RESCHK TO COMPUTE MAX-NORM AND 2-NORM OF RESIDUAL

CALL reschk(n, ia, ja, a, b, x)

! Now change the solution vector to all 1's
WRITE(*, *) 'Solution vector changed to ones'
x = 1.0_dp
CALL reschk(n, ia, ja, a, b, x)

STOP
5000 FORMAT (' IERR = ', i10, '  OK if positive')


CONTAINS


SUBROUTINE genprb(n, ia, ja, a, b, rhs)

!  GENPRB SETS UP THE MATRIX AND RIGHT HAND SIDE FROM THE DATA ON CARDS.
!  THE PROBLEM IS ONE FROM THE USGS.

INTEGER, INTENT(IN)     :: n
INTEGER, INTENT(OUT)    :: ia(:)
INTEGER, INTENT(OUT)    :: ja(:)
REAL (dp), INTENT(OUT)  :: a(:)
REAL (dp), INTENT(OUT)  :: b(:)
REAL (dp), INTENT(OUT)  :: rhs(:)

INTEGER    :: i, iaptr, ipart, j, k, kmin, kmax
REAL (dp)  :: rhsi

OPEN(UNIT=8, FILE='test2.dat', STATUS='OLD')
kmin = 1
DO  i = 1, n
  kmax = kmin + 38
  READ (8, *) b(kmin:kmax)
  kmin = kmax + 1
END DO

ia(1) = 1
iaptr = 1
ipart = 0
DO  i = 1, n
  rhsi = 0.0_dp
  DO  j = 1, 39
    k = i - 20 + j
    IF (k > 0) THEN
      IF (k <= n) THEN
        ipartj = ipart + j
        IF (b(ipartj) /= 0.0_dp) THEN
          a(iaptr) = b(ipartj)
          ja(iaptr) = k
! The line below is probably NOT what was intended, and has been replaced.
!         rhsi = rhsi + b(iaptr)
          rhsi = rhsi + a(iaptr)
          iaptr = iaptr + 1
        END IF
      END IF
    END IF
  END DO
  ia(i+1) = iaptr
  rhs(i) = rhsi
  ipart = ipart + 39
END DO
RETURN

END SUBROUTINE genprb

END PROGRAM PIVCHK
